<?php

namespace Drupal\ohdear_integration\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Cache\CacheableJsonResponse;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Cache\CacheableResponseInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\Core\Render\RenderContext;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\ohdear_integration\OhDearHealthcheckGenerator;
use Drupal\ohdear_integration\OhDearSdkService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Cache\RefinableCacheableDependencyInterface;

/**
 * Returns responses for OhDear Integration routes.
 */
class OhDearIntegrationController extends ControllerBase {
  use LoggerChannelTrait;

  /**
   * Healthcheck generator service.
   *
   * @var \Drupal\ohdear_integration\OhDearHealthcheckGenerator
   */
  protected $ohDearGenerator;

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The core renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The OhDear service development kit and helper service.
   *
   * @var \Drupal\ohdear_integration\OhDearSdkService
   */
  protected $ohDearSdk;

  /**
   * Cache max age ohdear healthcheck json endpoint configuration.
   *
   * @var int|null
   */
  protected $cacheMaxAge;

  /**
   * Constructs OhDear integration controller.
   *
   * @param \Drupal\ohdear_integration\OhDearHealthcheckGenerator $ohdear_generator
   *   The ohdear helper objects factory service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger channel factory service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack service.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The core renderer service.
   * @param \Drupal\ohdear_integration\OhDearSdkService $ohdear_sdk
   *   The ohdear SDK service.
   */
  public function __construct(OhDearHealthcheckGenerator $ohdear_generator, LoggerChannelFactoryInterface $logger_factory, RequestStack $request_stack, RendererInterface $renderer, OhDearSdkService $ohdear_sdk) {
    $this->ohDearGenerator = $ohdear_generator;
    $this->setLoggerFactory($logger_factory);
    $this->requestStack = $request_stack;
    $this->renderer = $renderer;
    $this->ohDearSdk = $ohdear_sdk;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): OhDearIntegrationController {
    return new static(
      $container->get('ohdear_healthcheck.generator'),
      $container->get('logger.factory'),
      $container->get('request_stack'),
      $container->get('renderer'),
      $container->get('ohdear_sdk')
    );
  }

  /**
   * Builds the response.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   Response.
   */
  public function buildJson() {
    // Render response in context to avoid metadata leaks. Some monitoring
    // plugins are rendered on sensor run and cached at that time as well. That
    // metadata is hard to access, so we need rendering in context detour.
    $context = new RenderContext();
    $response = $this->renderer->executeInRenderContext($context, function () {
      if ($this->access($this->currentUser())->isForbidden()) {
        $json_response = new CacheableJsonResponse(
          '{"error": "Access denied!"}',
          403,
          [
            'Cache-Control' => 'private, no-cache, no-store, must-revalidate',
          ],
          TRUE
        );
      }
      else {
        $data = $this->ohDearGenerator->getData();
        $max_age = $this->getCacheMaxAge();
        $json_response = new CacheableJsonResponse(
          $data,
          200,
          [
            'Cache-Control' => "public,max-age=$max_age,s-maxage=$max_age",
          ],
          TRUE
        );
      }
      // Cache is mostly set outside of context.
      $this->addCacheToResponse($json_response);
      $cacheable_metadata = $json_response->getCacheableMetadata();
      $this->refineCacheMetadata($cacheable_metadata);
      $json_response->addCacheableDependency($cacheable_metadata);
      return $json_response;
    });
    $this->addCacheToResponse($response);
    // If there is metadata left on the context, apply it on the response.
    if (!$context->isEmpty()) {
      if ($metadata = $context->pop()) {
        $this->refineCacheMetadata($metadata);
        $response->addCacheableDependency($metadata);
      }
    }
    return $response;
  }

  /**
   * Add cache configuration to response.
   *
   * @param CacheableResponseInterface $response
   *   Response to add cache to.
   *
   * @return CacheableResponseInterface
   *   Response with cache added.
   */
  protected function addCacheToResponse(CacheableResponseInterface $response) {
    $cache_max_age = $this->getCacheMaxAge();
    if (!$cache_max_age || !$response->isCacheable()) {
      // 403 response is not cacheable.
      $response->headers->set('Cache-Control', 'private,no-cache,must-revalidate');
    }
    else {
      $response->headers->set('Cache-Control', "public,max-age=$cache_max_age,s-maxage=$cache_max_age");
    }
    $cacheable_metadata = $response->getCacheableMetadata();
    $this->refineCacheMetadata($cacheable_metadata);
    $response->addCacheableDependency($cacheable_metadata);
    return $response;
  }

  /**
   * Get cache max age configuration
   *
   * @return mixed
   */
  protected function getCacheMaxAge() {
    if (!isset($this->cacheMaxAge)) {
      $this->cacheMaxAge = $this->ohDearSdk->getHealthcheckCacheMaxAge();
    }
    return $this->cacheMaxAge;
  }

  /**
   * Access result callback.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   Determines the access to controller.
   */
  public function access(AccountInterface $account) {
    $current_request = $this->requestStack->getCurrentRequest();
    $ohdear_header_secret = $current_request->headers->get('oh-dear-health-check-secret') ?? $current_request->query->get('oh-dear-health-check-secret');
    $ohdear_healthcheck_secret = $this->ohDearSdk->getHealthcheckSecret();
    if (($ohdear_header_secret && $ohdear_header_secret === $ohdear_healthcheck_secret)
      || $account->hasPermission('monitoring reports')) {
      $access_result = AccessResult::allowed();
    }
    else {
      $access_result = AccessResult::forbidden();
      $this->getLogger('ohdear_integration')->notice('Unauthorized access to monitoring results denied.');
    }
    $this->refineCacheMetadata($access_result);
    return $access_result;
  }

  /**
   * Adds cache metadata related to the healthheck.
   *
   * @param \Drupal\Core\Cache\RefinableCacheableDependencyInterface $cacheable_dependency
   *   An object implementing RefinableCacheableDependencyInterface.
   */
  protected function refineCacheMetadata(RefinableCacheableDependencyInterface $cacheable_dependency) {
    $cacheable_dependency->addCacheContexts([
      'url.query_args:oh-dear-health-check-secret',
      'headers:oh-dear-health-check-secret',
      'user.roles',
    ]);
    $cacheable_dependency->addCacheTags(['monitoring_sensor_result']);
    $cacheable_dependency->setCacheMaxAge($this->getCacheMaxAge() ?? 0);
  }

}
